using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;

namespace DotNetZip
{
    public class HTTPRangeStream : Stream
    {
        private string url;
        private long length = 0;
        private long position = 0;
        private long totalReadBytes = 0;
        private int totalReadTimes = 0;

        public HTTPRangeStream(string URL)
        {
            url = URL;
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(url);
            HttpWebResponse result = (HttpWebResponse)request.GetResponse();
            length = result.ContentLength;
        }

        public long TotalReadBytes
        {
            get { return totalReadBytes; }
        }

        public long TotalReadTimes
        {
            get { return totalReadTimes; }
        }

        public override bool CanRead
        {
            get { return true; }
        }

        public override bool CanSeek
        {
            get { return true; }
        }

        public override bool CanTimeout
        {
            get
            {
                return base.CanTimeout;
            }
        }

        public override bool CanWrite
        {
            get { return false; }
        }

        public override long Length
        {
            get { return length; }
        }

        public override long Position
        {
            get
            {
                return position;
            }
            set
            {
                position = value;
            }
        }

        public override void Flush()
        {            
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            //todo:
            switch (origin)
            {
                case SeekOrigin.Begin:
                    position = offset;
                    break;
                case SeekOrigin.Current:
                    position += offset;
                    break;
                case SeekOrigin.End:
                    position = Length + offset;
                    break;
                default:
                    break;
            }
            return Position;
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(url);
            request.AddRange(Convert.ToInt32(position), Convert.ToInt32(position) + count);
            HttpWebResponse result = (HttpWebResponse)request.GetResponse();
            using (Stream stream = result.GetResponseStream())
            {
                stream.Read(buffer, offset, count);
                stream.Close();
            }
            totalReadBytes += count;
            totalReadTimes++;
            Position += count;
            return count;
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
        }

        public override void SetLength(long value)
        {
            
        }
    }
}
